#ifndef _BIOLOID_REGISTERS_H
#define _BIOLOID_REGISTERS_H

#ifdef __cplusplus
extern "C" {
#endif

#define DEVICE_MODEL_OFFSET      ((unsigned short int)0x0000)
#define FIRMWARE_VERSION_OFFSET  ((unsigned short int)0x0002)
#define DEVICE_ID_OFFSET         ((unsigned short int)0x0003)
#define BAUDRATE_OFFSET          ((unsigned short int)0x0004)
#define RETURN_DELAY_OFFSET      ((unsigned short int)0x0005)
#define RETURN_LEVEL_OFFSET      ((unsigned short int)0x0010)

#define LAST_EEPROM_OFFSET       ((unsigned short int)0x0020)

typedef enum {
  BIOLOID_MODEL_NUMBER_L          = DEVICE_MODEL_OFFSET,
  BIOLOID_MODEL_NUMBER_H          = DEVICE_MODEL_OFFSET+1,
  BIOLOID_VERSION                 = FIRMWARE_VERSION_OFFSET,
  BIOLOID_ID                      = DEVICE_ID_OFFSET,
  BIOLOID_BAUD_RATE               = BAUDRATE_OFFSET,
  BIOLOID_RETURN_DELAY_TIME       = RETURN_DELAY_OFFSET,
  BIOLOID_RETURN_LEVEL            = RETURN_LEVEL_OFFSET,
  BIOLOID_USER1_LED_CNTRL         = 0x20, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2  | bit 1 |      bit 0
                                          //       |       |       |       | blink | toggle | value | internally used
  BIOLOID_USER1_LED_PERIOD_L      = 0x21,
  BIOLOID_USER1_LED_PERIOD_H      = 0x22,     
  BIOLOID_USER2_LED_CNTRL         = 0x23, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2  | bit 1 |      bit 0
                                          //       |       |       |       | blink | toggle | value | internally used
  BIOLOID_USER2_LED_PERIOD_L      = 0x24,
  BIOLOID_USER2_LED_PERIOD_H      = 0x25,     
  BIOLOID_RXD_LED_CNTRL           = 0x26, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2  | bit 1 |      bit 0
                                          //       |       |       |       | blink | toggle | value | internally used
  BIOLOID_RXD_LED_PERIOD_L        = 0x27,
  BIOLOID_RXD_LED_PERIOD_H        = 0x28,     
  BIOLOID_TXD_LED_CNTRL           = 0x29, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2  | bit 1 |      bit 0
                                          //       |       |       |       | blink | toggle | value | internally used
  BIOLOID_TXD_LED_PERIOD_L        = 0x2A,
  BIOLOID_TXD_LED_PERIOD_H        = 0x2B,     
  BIOLOID_USER_PB_CNTRL           = 0x2C, // bit 7 | bit 6 | bit 5 | bit 4 |     bit 3      |       bit 2      | bit 1 |      bit 0
                                          //       |       |       |       | interrupt flag | enable interrupt | value | internally used
  BIOLOID_RESET_PB_CNTRL          = 0x2D, // bit 7 | bit 6 | bit 5 | bit 4 |     bit 3      |       bit 2      | bit 1 |      bit 0
                                          //       |       |       |       | interrupt flag | enable interrupt | value | internally used
  BIOLOID_START_PB_CNTRL          = 0x2E, // bit 7 | bit 6 | bit 5 | bit 4 |     bit 3      |       bit 2      | bit 1 |      bit 0
                                          //       |       |       |       | interrupt flag | enable interrupt | value | internally used
  BIOLOID_MODE_PB_CNTRL           = 0x2F, // bit 7 | bit 6 | bit 5 | bit 4 |     bit 3      |       bit 2      | bit 1 |      bit 0
                                          //       |       |       |       | interrupt flag | enable interrupt | value | internally used
} bioloid_registers;

#define      GPIO_BASE_ADDRESS    0x20
#define      GPIO_MEM_LENGTH      16
#define      GPIO_INT_USED        0x01
#define      GPIO_VALUE           0x02
#define      GPIO_TOGGLE          0x04
#define      GPIO_BLINK           0x08
#define      GPIO_INT_EN          0x04
#define      GPIO_INT_FLAG        0x08

#ifdef __cplusplus
}
#endif

#endif
