#include "bioloid_dyn_slave.h"
#include "bioloid_time.h"
#include "usart3.h"
#include "ram.h"
#include "eeprom.h"
#include "gpio.h"

/* external interrupt pin */
#define     DYN_SLAVE_EXT_INT_PIN             GPIO_PIN_8
#define     DYN_SLAVE_EXT_INT_GPIO_PORT       GPIOE
#define     DYN_SLAVE_EXT_INT_EN_GPIO_CLK     __HAL_RCC_GPIOE_CLK_ENABLE()

/* private variables */
TDynamixelSlave bioloid_dyn_slave;
TTime bioloid_dyn_slave_timer;
TComm bioloid_dyn_slave_comm;
UART_InitTypeDef bioloid_comm_init;
uint8_t bioloid_slave_int_state;

// private functions
unsigned char bioloid_on_read(unsigned short int address,unsigned short int length,unsigned char *data)
{
  unsigned char error;
  /* dynamixel slave internal operation registers */
  error=ram_read_table(address,length,data);  

  return error;
}

unsigned char bioloid_on_write(unsigned short int address,unsigned short int length,unsigned char *data)
{
  unsigned char i,j;

  /* dynamixel slave internal operation registers */
  if(ram_in_range(BIOLOID_ID,address,length))
  {
    dyn_slave_set_address(&bioloid_dyn_slave,data[BIOLOID_ID-address]);
    ram_write_byte(BIOLOID_ID,data[BIOLOID_ID-address]);
  }
  if(ram_in_range(BIOLOID_BAUD_RATE,address,length))
  {
    bioloid_comm_init.BaudRate=2000000/(data[BIOLOID_BAUD_RATE-address]+1);
    usart3_config(&bioloid_dyn_slave_comm,&bioloid_comm_init);
    ram_write_byte(BIOLOID_BAUD_RATE,data[BIOLOID_BAUD_RATE-address]);
  }
  if(ram_in_range(BIOLOID_RETURN_DELAY_TIME,address,length))
  {
    dyn_slave_set_return_delay(&bioloid_dyn_slave,data[BIOLOID_RETURN_DELAY_TIME-address]);
    ram_write_byte(BIOLOID_RETURN_DELAY_TIME,data[BIOLOID_RETURN_DELAY_TIME-address]);
  }
  if(ram_in_range(BIOLOID_RETURN_LEVEL,address,length))
  {
    dyn_slave_set_return_level(&bioloid_dyn_slave,data[BIOLOID_RETURN_LEVEL-address]);
    ram_write_byte(BIOLOID_RETURN_LEVEL,data[BIOLOID_RETURN_LEVEL-address]);
  }
  // GPIO commands
  if(ram_in_window(GPIO_BASE_ADDRESS,GPIO_MEM_LENGTH,address,length))
    gpio_process_cmd(address,length,data);
  // write eeprom
  for(i=address,j=0;i<LAST_EEPROM_OFFSET && i<(address+length);i++,j++)
    EE_WriteVariable(i,data[j]);

  return 0x00;
}

// public functions
void bioloid_dyn_slave_init(void)
{
  TUSART_IRQ_Priorities priorities;
  GPIO_InitTypeDef GPIO_InitStructure;

  // initialize private variables
  bioloid_slave_int_state=0;

  // initialize timer structure
  time_init(&bioloid_dyn_slave_timer,bioloid_time_get_counts_per_us(),bioloid_time_get_counts);

  /* initialize the comm object */
  comm_init(&bioloid_dyn_slave_comm,0x01,&bioloid_dyn_slave_timer);
  bioloid_comm_init.BaudRate     = 921600;
  bioloid_comm_init.WordLength   = UART_WORDLENGTH_8B;
  bioloid_comm_init.StopBits     = UART_STOPBITS_1;
  bioloid_comm_init.Parity       = UART_PARITY_NONE;
  bioloid_comm_init.Mode         = UART_MODE_TX_RX;
  bioloid_comm_init.HwFlowCtl    = UART_HWCONTROL_NONE;
  bioloid_comm_init.OverSampling = UART_OVERSAMPLING_16;

  priorities.irq_priority=0;
  priorities.irq_subpriority=0;
  priorities.dma_rx_priority=1;
  priorities.dma_rx_subpriority=1;
  priorities.dma_tx_priority=1;
  priorities.dma_tx_subpriority=0;

  usart3_init(&bioloid_dyn_slave_comm,&bioloid_comm_init,&priorities);
  dyn_slave_init(&bioloid_dyn_slave,&bioloid_dyn_slave_comm,ram_data[BIOLOID_ID]);
  bioloid_dyn_slave.on_read=bioloid_on_read;
  bioloid_dyn_slave.on_write=bioloid_on_write;
  dyn_slave_set_return_delay(&bioloid_dyn_slave,ram_data[BIOLOID_RETURN_DELAY_TIME]);
  dyn_slave_set_return_level(&bioloid_dyn_slave,ram_data[BIOLOID_RETURN_LEVEL]);

  /* initialize the external interrupt pin */
  DYN_SLAVE_EXT_INT_EN_GPIO_CLK;
  GPIO_InitStructure.Pin       = DYN_SLAVE_EXT_INT_PIN;
  GPIO_InitStructure.Mode      = GPIO_MODE_OUTPUT_OD;
  GPIO_InitStructure.Pull      = GPIO_NOPULL;
  GPIO_InitStructure.Speed     = GPIO_SPEED_HIGH;
  HAL_GPIO_Init(DYN_SLAVE_EXT_INT_GPIO_PORT, &GPIO_InitStructure);
}

void bioloid_dyn_slave_set_int(void)
{
  HAL_GPIO_WritePin(DYN_SLAVE_EXT_INT_GPIO_PORT,DYN_SLAVE_EXT_INT_PIN,GPIO_PIN_RESET);
  bioloid_slave_int_state=1;
}

void bioloid_dyn_slave_clear_int(void)
{
  HAL_GPIO_WritePin(DYN_SLAVE_EXT_INT_GPIO_PORT,DYN_SLAVE_EXT_INT_PIN,GPIO_PIN_SET);
  bioloid_slave_int_state=0;
}

uint8_t bioloid_dyn_slave_is_int_set(void)
{
  return bioloid_slave_int_state;
}
