#include "ram.h"
#include "eeprom.h"

uint8_t ram_data[RAM_SIZE];

void ram_init(void)
{
  uint16_t eeprom_data,i;

  for(i=0;i<RAM_SIZE;i++)
    ram_data[i]=0x00;
  // read contents from EEPROM to RAM
  if(EE_ReadVariable(DEVICE_MODEL_OFFSET,&eeprom_data)==0)
    ram_data[DEVICE_MODEL_OFFSET]=(uint8_t)(eeprom_data&0x00FF);
  if(EE_ReadVariable(DEVICE_MODEL_OFFSET+1,&eeprom_data)==0)
    ram_data[DEVICE_MODEL_OFFSET+1]=(uint8_t)(eeprom_data&0x00FF);
  if(EE_ReadVariable(FIRMWARE_VERSION_OFFSET,&eeprom_data)==0)
    ram_data[FIRMWARE_VERSION_OFFSET]=(uint8_t)eeprom_data;
  if(EE_ReadVariable(DEVICE_ID_OFFSET,&eeprom_data)==0)
    ram_data[DEVICE_ID_OFFSET]=(uint8_t)eeprom_data;
  if(EE_ReadVariable(BAUDRATE_OFFSET,&eeprom_data)==0)
    ram_data[BAUDRATE_OFFSET]=(uint8_t)eeprom_data;
  if(EE_ReadVariable(RETURN_DELAY_OFFSET,&eeprom_data)==0)
    ram_data[RETURN_DELAY_OFFSET]=(uint8_t)eeprom_data;
  if(EE_ReadVariable(RETURN_LEVEL_OFFSET,&eeprom_data)==0)
    ram_data[RETURN_LEVEL_OFFSET]=(uint8_t)eeprom_data;
}

inline void ram_read_byte(uint8_t address,uint8_t *data)
{
  (*data)=ram_data[address];
}

inline void ram_read_word(uint8_t address, uint16_t *data)
{
  (*data)=ram_data[address];
  (*data)+=ram_data[address+1]*256;
}

uint8_t ram_read_table(uint8_t address, uint8_t length,uint8_t *data)
{
  uint8_t i;

  if((address+length)<=(RAM_SIZE-1))
  {
    for(i=0;i<length;i++)
      data[i]=ram_data[address+i];
    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_ADDRESS;
}

uint8_t ram_set_bit(uint8_t address, uint8_t bit)
{
  if(bit>=0 && bit<8)
  {
    ram_data[address]|=(0x01<<bit);
    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_BIT;
}

uint8_t ram_clear_bit(uint8_t address, uint8_t bit)
{
  if(bit>=0 && bit<8)
  {
    ram_data[address]&=(~(0x01<<bit));
    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_BIT;
}

uint8_t ram_write_byte(uint8_t address, uint8_t data)
{
  ram_data[address]=data;

  return RAM_SUCCESS;
}

uint8_t ram_write_word(uint8_t address, uint16_t data)
{
  if(address < (RAM_SIZE-1))
  {
    ram_data[address]=data%256;
    ram_data[address+1]=data/256;

    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_ADDRESS;  
}

uint8_t ram_write_table(uint8_t address, uint8_t length,uint8_t *data)
{
  uint8_t i;

  if((address+length)<RAM_SIZE)
  {
    for(i=0;i<length;i++)
      ram_data[address+i]=data[i];
    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_ADDRESS;
}

inline uint8_t ram_in_range(uint8_t reg,uint8_t address,uint8_t length)
{
  if(reg>=address && reg<(address+length))
    return 0x01;
  else
    return 0x00;
}

inline uint8_t ram_in_window(uint8_t start_reg,uint8_t reg_length,uint8_t start_address,uint8_t address_length)
{
  if((start_reg>=start_address && start_reg<(start_address+address_length)) || 
     ((start_reg+reg_length)>=start_address && (start_reg+reg_length)<(start_address+address_length)))
    return 0x01;
  else
    return 0x00;
}
